from dataclasses import dataclass
from pathlib import Path
from typing import Optional

import numpy as np
from matplotlib import pyplot as plt

from hpcflow.sdk.core.parameters import ParameterValue
from matflow.param_classes.orientations import Orientations


@dataclass
class MicrostructureSeeds(ParameterValue):
    _typ = "microstructure_seeds"

    position: np.ndarray
    box_size: np.ndarray
    phase_label: str
    orientations: Optional[Orientations] = None
    random_seed: Optional[int] = None

    def __post_init__(self):
        self.box_size = np.asarray(self.box_size)
        self.position = np.asarray(self.position)
        if self.orientations and not isinstance(self.orientations, Orientations):
            self.orientations = Orientations(**self.orientations)

    @classmethod
    def from_JSON_like(cls, position, orientations=None, **kwargs):
        """For custom initialisation via YAML or JSON."""
        # TODO: is this needed?
        if orientations:
            orientations = Orientations.from_JSON_like(**orientations)
        return cls(position=np.asarray(position), orientations=orientations, **kwargs)

    @property
    def num_seeds(self):
        return self.position.shape[0]

    @classmethod
    def from_random(
        cls,
        num_seeds,
        box_size,
        phase_label,
        random_seed=None,
        orientations=None,
    ):
        # TODO: ensure unique seeds points wrt to grid cells
        box_size = np.asarray(box_size)
        rng = np.random.default_rng(seed=random_seed)
        position = rng.random((num_seeds, box_size.size)) * box_size
        return cls(
            position=position,
            box_size=box_size,
            phase_label=phase_label,
            orientations=orientations,
            random_seed=random_seed,
        )

    @classmethod
    def from_file(
        cls,
        path,
        box_size,
        phase_label,
        number=None,
        start_index=0,
        delimiter=" ",
    ):
        data = []
        with Path(path).open("rt") as fh:
            for idx, line in enumerate(fh):
                line = line.strip()
                if not line or idx < start_index:
                    continue
                elif len(data) < number if number is not None else True:
                    data.append([float(i) for i in line.split(delimiter)])
        if number is not None and len(data) < number:
            raise ValueError("Not enough seed points in the file.")

        return cls(
            position=np.asarray(data),
            box_size=box_size,
            phase_label=phase_label,
        )

    def show(self):
        fig = plt.figure()
        ax = fig.add_subplot(projection="3d")
        ax.scatter(
            self.position[:, 0],
            self.position[:, 1],
            self.position[:, 2],
        )
        plt.show()



from hpcflow.sdk import app as sdk_app
from hpcflow.sdk.config import ConfigOptions

from matflow._version import __version__


# provide access to app attributes:
__getattr__ = sdk_app.get_app_attribute

# ensure docs/help can see dynamically loaded attributes:
__all__ = sdk_app.get_app_module_all()
__dir__ = sdk_app.get_app_module_dir()

# set app-level config options:
config_options = ConfigOptions(
    directory_env_var="MATFLOW_CONFIG_DIR",
    default_directory="~/.matflow-new",
    sentry_DSN="https://2463b288fd1a40f4bada9f5ff53f6811@o1180430.ingest.sentry.io/6293231",
    sentry_traces_sample_rate=1.0,
    sentry_env="main" if "a" in __version__ else "develop",
    default_known_configs_dir="github://hpcflow:matflow-configs@main",
)

# load built in template components:
template_components = sdk_app.App.load_builtin_template_component_data("matflow.data")

# initialise the App object:
app: sdk_app.App = sdk_app.App(
    name="MatFlow",
    version=__version__,
    module=__name__,
    docs_import_conv="mf",
    description="Materials science workflow manager",
    template_components=template_components,
    scripts_dir="data.scripts",  # relative to root package
    workflows_dir="data.workflows",  # relative to root package
    config_options=config_options,
    docs_url="https://docs.matflow.io/stable",
)  #: |app|

# defer import to allow us to use the app logger in the ParameterValue classes:
from matflow.param_classes import *



from matflow.param_classes import LoadCase


def test_single_multistep_uniaxial():
    args = {
        "num_increments": 200,
        "total_time": 100,
        "direction": "x",
        "target_def_grad_rate": 1e-3,
    }
    lc1 = LoadCase.uniaxial(**args)
    lc2 = LoadCase.multistep(steps=[{"type": "uniaxial", **args}])
    assert lc1 == lc2



from pathlib import Path
from typing import Dict
from damask_parse.readers import read_HDF5_file


def extract_damask_HDF5(damask_hdf5_file: Path, VE_response_data: Dict):
    VE_response = read_HDF5_file(hdf5_path=damask_hdf5_file, **VE_response_data)
    return VE_response
